using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;

namespace Feel
{
    public class GlassBodyTracker : MVRScript
    {
        #region PluginInfo    
        public string pluginAuthor = "Feel";
        public string pluginName = "GlassBodyTracker";
        public string pluginVersion = "1";
        public string pluginDate = "11/20" + "/2022";
        public string pluginDescription = @"Glass Body Target";
        #endregion

        #region Vars    

        protected bool first_loading = false;

        protected JSONStorableBool IsActive;
        protected JSONStorableFloat xRotation;
        protected JSONStorableFloat yRotation;
        protected JSONStorableFloat zRotation;
        protected JSONStorableStringChooser _targetSelectJSON;
        protected List<String> _targetBodyChoices;
        protected UIDynamicPopup _targetDisplayPopup;

        public Transform ObjectRoot;
        public Transform TargetRoot;
        public Transform TargetGenRoot;

        private Atom _thisAtom;
        private FreeControllerV3 _thisControl;

        private bool editorInitialized = false;
        private int initLoops = 0;
        private int maxInitLoops = 1500; // 3600 for one minute

        #endregion

        public override void Init()
        {
            try
            {
                if (containingAtom.type != "CustomUnityAsset")
                {
                    SuperController.LogError("GlassBodyTracker - Please add this script on a Custom Unity Asset ( script disabled).");
                    return;
                }

                _thisAtom = containingAtom;
                _thisControl = _thisAtom.freeControllers.First(fc => fc.name == "control");

                IsActive = new JSONStorableBool("Active", false, SetTargetActive);
                RegisterBool(IsActive);
                UIDynamicToggle toggle2 = CreateToggle(IsActive, false);
                toggle2.label = "Active On Off";
                toggle2.height = 80;

                _targetBodyChoices = new List<string>();

                var btn = CreateButton("Target Update", false);

                btn.button.onClick.AddListener(() =>
                {
                    OnTargetListUpdate();
                });

                _targetSelectJSON = new JSONStorableStringChooser("Target Choice", null, null, "", OnTargetSelect);
                _targetSelectJSON.storeType = JSONStorableParam.StoreType.Physical;
                RegisterStringChooser(_targetSelectJSON);

                _targetDisplayPopup = CreateScrollablePopup(_targetSelectJSON, false);
                _targetDisplayPopup.popupPanelHeight = 300f;
                _targetDisplayPopup.popup.onOpenPopupHandlers += SyncOrificeChoices;
                _targetDisplayPopup.label = "Target:";

                _targetSelectJSON.choices = _targetBodyChoices;

                xRotation = new JSONStorableFloat("x rotation", 0f, OnChangeXRotation, -180f, 180f);
                RegisterFloat(xRotation);
                CreateSlider(xRotation, false);

                yRotation = new JSONStorableFloat("y rotation", 0f, OnChangeXRotation, -180f, 180f);
                RegisterFloat(yRotation);
                CreateSlider(yRotation, false);

                zRotation = new JSONStorableFloat("z rotation", 0f, OnChangeXRotation, -180f, 180f);
                RegisterFloat(zRotation);
                CreateSlider(zRotation, false);


                // A tiny bit of help to give some info to your fellow creator and/or player
                JSONStorableString helpText = new JSONStorableString("About",
                    "<color=#000><size=35><b>About</b></size></color>\n\n" +
                    "<color=#333>" +
                    "Glass Body\n\n" +
                    "</color>");
                UIDynamic helpTextfield = CreateTextField(helpText, true);
                helpTextfield.height = 500.0f;

            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // Update is called with each rendered frame by Unity
        void Update()
        {
            // I don't know why the fuck this asset does not get the children properly during the loading process, so I'm checking the singleton
            // In reality, that's even cleaner to do it that way
            if (initLoops < maxInitLoops && editorInitialized == false && !SuperController.singleton.isLoading)
            {
                initLoops++;
                InitEditor();
            }

            if (initLoops == maxInitLoops)
            {
                SuperController.LogError("GlassBodyTracker : This CUA could not be initialized properly. Please select a compatible object and reload the script.");
                initLoops++; // To avoid spamming the log with the error
            }

            // first set.
            if(first_loading == false)
            {
                first_loading = true;
            }
            
            
        }

        void FixedUpdate()
        {
            if (TargetRoot && TargetGenRoot)
            {
                if(IsActive.val)
                {
                    _thisControl.transform.position = TargetGenRoot.position;

                    Vector3 ro = TargetGenRoot.rotation.eulerAngles;
                    //ro.x -= 360f * xRotation.val;
                    //ro.y += 360f * yRotation.val;
                    //ro.z += 360f * zRotation.val;

                    //SuperController.LogMessage(xRotation.val + " - " + ro.x);

                    //_thisControl.transform.rotation = Quaternion.Euler(ro);
                    _thisControl.transform.rotation = TargetGenRoot.rotation;
                    _thisControl.transform.Rotate(xRotation.val - 90f, yRotation.val, zRotation.val);
                }
            }
        }

        void OnDestroy()
        {
            
        }

        private bool InitEditor()
        {
            if (editorInitialized == true) return true;

            //ObjectRoot = getObjectRoot(containingAtom.reParentObject, "GlassBody");
            //if (ObjectRoot == null)
            //{
            //    return false;
            //}

            //TargetRoot = getChildRoot(containingAtom.reParentObject, "Armature");

            OnTargetListUpdate();

            // I found my transforms, now the rest
            if (TargetRoot && TargetGenRoot)
            {
                editorInitialized = true;

                if (_targetBodyChoices.Count > 0)
                {
                    OnSelectTargetObj(_targetSelectJSON.val);
                }

                // Everything is initialized, triggering the callbacks to init the door rotation saved in the scene file
                // I'm just quickly fixing the script for VamTastic, I'm not gonna refactor everything, so in reality,
                // the float val param of the callbacks is useless

                return true;
            }
            else
            {
                SuperController.LogError("GlassBodyTracker : your is missing mandatory GlassBody.");
                initLoops = maxInitLoops; // We don't have what we need, we consider the initialization failed
                return false;
            }

            return false;
        }

        protected void SetTargetActive(JSONStorableBool jb)
        {
            if(!jb.val)
            {
                

            }
        }

        void OnChangeXRotation(float value)
        {

        }

        protected void SyncOrificeChoices()
        {
            //SuperController.LogMessage(">> SyncOrificeChoices ");
            //UpdateOrificeChoices();
        }

        protected void OnTargetSelect(string target_name)
        {

            if (_targetBodyChoices.Count > 0) OnSelectTargetObj(target_name);
            //SuperController.LogMessage(">> OnTargetSelect " + target_name);
            
        }

        private void OnTargetListUpdate()
        {
            List<string> uids = SuperController.singleton.GetAtomUIDs();
            
            _targetBodyChoices = new List<string>();

            foreach (string uid in uids)
            {
                Atom atom = SuperController.singleton.GetAtomByUid(uid);

                Transform[] trans = atom.GetComponentsInChildren<Transform>();
                bool in_glassbody = false;
                foreach (Transform tran in trans)
                {
                    if (tran.name.StartsWith("GlassBody"))
                    {
                        TargetRoot = tran;
                        in_glassbody = true;
                    }
                    if (in_glassbody && tran.name.StartsWith("CumEffect"))
                    {
                        TargetGenRoot = tran;
                        
                        _targetBodyChoices.Add(uid);
                        //SuperController.LogMessage("--- " + tran.name);
                    }
                    
                }

            }

            _targetSelectJSON.choices = _targetBodyChoices;
        }

        private void OnSelectTargetObj(string uid)
        {
            Atom atom = SuperController.singleton.GetAtomByUid(uid);
            Transform[] trans = atom.GetComponentsInChildren<Transform>();
            foreach (Transform tran in trans)
            {
                if (tran.name.StartsWith("CumEffect"))
                {
                    TargetGenRoot = tran;
                    //SuperController.LogMessage("---------------- " + uid);
                }
            }
        }
    }

}